@@ -7,7 +7,6 @@ package XML::RSS::Parser::Element;
 
 use strict;
 use vars qw( $VERSION );
-$VERSION = '2.01';
 
 use Class::XPath 1.4
      get_name => '_xpath_name',
@@ -91,9 +90,11 @@ sub children {
 }
 
 sub children_names { keys %{ $_[0]->{__children} }; }
-sub attribute { $_[0]->{attributes}->{ $_[1] } = $_[2]; $_[0]->{attributes}->{ $_[1] } } 
+sub attribute { $_[0]->{attributes}->{ $_[1] } = $_[2] if $_[2]; $_[0]->{attributes}->{ $_[1] } } 
 sub attributes { $_[0]->{attributes}=$_[1] if $_[1]; $_[0]->{attributes}; };
 
+*query = \&match;
+
 sub _xpath_name {
 	my $name = $_[1] || $_[0]->{name}; # for reuse.
 	my $ns;
@@ -105,19 +106,26 @@ sub _xpath_name {
 sub _xpath_attribute_names { 
 	my $self = shift;
 	return unless $self->{attributes};
- 	map { $self->_xpath_name($_) } keys %{ $self->{attributes} };
+    my @a;
+    foreach (keys %{ $self->{attributes} }) {
+            my($ns,$name) = ($_=~ m!(.+?)(\w+)$!); 
+            push @a, join(':',$xpath_ns{$ns},$name);
+    }
+    @a;
 }
 
 sub _xpath_attribute {
 	my $self = shift;
 	my $name = shift;
+	my $ns = '';
 	if ( $name=~/(\w+):(\w+)/ ) {
-		$name = $1;
-		my $ns = $xpath_ns{ $2 };
-		$ns .= '/' unless ($ns=~m![/#]$!);
-		$name = "$ns$name";
+		$name = $2;
+		$ns = $xpath_prefix{$1};
+		$ns .=  '/' unless $ns=~m![/#]$!;
+	} else {
+	    ($ns = $self->name)=~ s/\w+$//;
 	}
-	$self->attribute($name);
+	$self->attribute("$ns$name");
 }
 
 1;
@@ -143,65 +151,81 @@ for more.
 
 =item $element->root( [$feed] )
 
-Returns a reference to the root element (typically a L<XML::RSS::Parser::Feed>. If an optional SCALAR 
-parameter is passed in the root element is set. Is undefined if not set.
+Returns a reference to the root element of the parse tree. A L<XML::RSS::Parser::Feed> 
+can be passed to optionally set the root element. The default is undefined.
 
 =item $element->parent( [$element] )
 
-Returns a reference to the parent element. If an optional SCALAR parameter is passed in the 
-parent element is set. Is undefined if not set.
+Returns a reference to the parent element. A L<XML::RSS::Parser::Element> object 
+or one of its subclasses can be passed to optionally set the parent. The 
+default is undefined.
 
 =item $element->name( [$extended_name] )
 
-Returns the name of the element as a SCALAR. This should by the fully namespace qualified (extended) 
-name of the element and not the QName or local part. Is undefined if not set. 
+Returns the name of the element as a SCALAR. This should by the fully namespace 
+qualified (extended)  name of the element and not the QName or local part. The 
+default is undefined. 
 
 =item $element->value( [$value] )
 
-Returns a reference to the parent element. If an optional SCALAR parameter is passed in the 
-parent element is set. Returns a null string if not set.
+Returns a reference to the value (text contents) of the element. If an optional 
+SCALAR parameter is passed in the value (text contents) is set. Returns a null 
+string if not set.
 
 =item $element->append_value( $value )
 
-Appends the value of the SCALAR parameter to the object's current value. A convenience method that is
-particularly helpful when working in L<XML::Parser> handlers.
+Appends the value of the SCALAR parameter to the object's current value. A 
+convenience method that is particularly helpful when working in 
+L<XML::Parser> handlers.
 
 =item $element->attribute($name [, $value] )
 
-Returns the value of an attribute specified by C<$name> as a SCALAR. If an optional second parameter 
-(C<$value>) is passed in the attribute is set.
+Returns the value of an attribute specified by C<$name> as a SCALAR. If 
+an optional second text parameter C<$value> is passed in the attribute is set.
 
 =item $element->attributes( [\%attributes] )
 
-Returns a HASH reference contain attributes and their values as key value pairs. An optional parameter
-of a HASH reference can be passed in to set multiple attributes. NOTE: When setting attributes with 
-this method, all existing attributes are overwritten irregardless of whether they are present in the
+Returns a HASH reference contain attributes and their values as key value pairs. 
+An optional parameter of a HASH reference can be passed in to set multiple 
+attributes. NOTE: When setting attributes with this method, all existing 
+attributes are overwritten irregardless of whether they are present in the
 hash being passed in.
 
 =item $element->child( [$extended_name] )
 
-Constructs and returns a new element object making the current element its parent. An optional 
-parameter representing the name of the new element object can be passed. This should be the fully 
-namespace qualified (extended) name and not the QName or local part.
+Constructs and returns a new element object making the current object 
+as its parent. An optional parameter representing the name of the new 
+element object can be passed. This should be the fully namespace 
+qualified (extended) name and not the QName or local part.
 
 =item $element->children( [$extended_name] )
 
-Returns any array of child elements to the object. An optional parameter can be passed in to return 
-element(s) with a specific name. If called in a SCALAR context it will return only the first element 
-with this name. If called in an ARRAY context the function returns all elements with this name. If 
-no elements exist as a child of the object, and undefined value is returned.
+Returns any array of child elements to the object. An optional parameter 
+can be passed in to return element(s) with a specific name. If called in 
+a SCALAR context it will return only the first element with this name. 
+If called in an ARRAY context the function returns all elements with 
+this name. If no elements exist as a child of the object, and undefined 
+value is returned.
 
 =item $element->children_names
 
 Returns an array contain the names of the objects children.
 
+=head2 XPath-esque Methods
+
+=item $element->query($xpath)
+
+Finds matching nodes using an XPath-esque query from anywhere in the 
+tree. See the L<Class::XPath> documentation for more information.
+
 =item $element->match($xpath)
 
-Finds matching nodes using an XPath-esque query from anywhere in the tree. See the L<Class::XPath> documentation for more information.
+Alias for the C<query> method. For compatability. C<query> is preferred.
 
 =item $element->xpath
 
-Returns an unique XPath string to the current node which can be used as an identifier.
+Returns a unique XPath string to the current node which can be used 
+as an identifier.
 
 =head1 SEE ALSO
 
@@ -209,12 +233,15 @@ L<XML::RSS::Parser>, L<XML::SimpleObject>, L<Class::XPath>
 
 =head1 LICENSE
 
-The software is released under the Artistic License. The terms of the Artistic License are described 
-at L<http://www.perl.com/language/misc/Artistic.html>.
+The software is released under the Artistic License. The terms of 
+the Artistic License are described at 
+L<http://www.perl.com/language/misc/Artistic.html>.
 
 =head1 AUTHOR & COPYRIGHT
 
-Except where otherwise noted, XML::RSS::Parser is Copyright 2003-4, Timothy Appnel, 
-cpan@timaoutloud.org. All rights reserved.
+Except where otherwise noted, XML::RSS::Parser::Element is Copyright 
+2003-4, Timothy Appnel, cpan@timaoutloud.org. All rights reserved.
 
 =cut
+
+=end
\ No newline at end of file
